#!/usr/bin/perl -w

use Test::More;
use File::Spec;
use File::Find;

use strict;

$| = 1;

eval "use Test::Pod 0.95";

if ($@) {
    plan 'skip_all', 'Test::Pod v0.95 required for testing POD';
} else {
    Test::Pod->import;
    my @files;
    my $blib = File::Spec->catfile(qw(blib lib));
    find( sub { push @files, $File::Find::name if /\.p(l|m|od)$/ }, $blib);
    plan 'tests', scalar @files;
    pod_file_ok($_) foreach (@files);
}

